@@ -1,13 +1,13 @@
-/* 
+/*
  *  IMS Open Corpus Workbench (CWB)
  *  Copyright (C) 1993-2006 by IMS, University of Stuttgart
  *  Copyright (C) 2007-     by the respective contributers (see file AUTHORS)
- * 
+ *
  *  This program is free software; you can redistribute it and/or modify it
  *  under the terms of the GNU General Public License as published by the
  *  Free Software Foundation; either version 2, or (at your option) any later
  *  version.
- * 
+ *
  *  This program is distributed in the hope that it will be useful, but
  *  WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
@@ -15,12 +15,13 @@
  *  WWW at http://www.gnu.org/copyleft/gpl.html).
  */
 
-#include "../cl/globals.h"
+#include <stdio.h>
+#include <string.h>
 #include <math.h>
 
-#include "../cl/macros.h"
-#include "../cl/attributes.h"
+#include "../cl/cl.h"
 
+#include "../cl/ui-helpers.h"
 
 #include "corpmanag.h"
 
@@ -36,18 +37,24 @@
 
 #include "targets.h"
 
-SearchStrategy string_to_strategy(char *s)
+SearchStrategy
+string_to_strategy(const char *s)
 {
-  if (s == NULL) 
+  if (!s)
     return SearchNone;
-  else if (strcasecmp(s, "leftmost") == 0)
+
+  else if (cl_streq_ci(s, "leftmost"))
     return SearchLeftmost;
-  else if (strcasecmp(s, "rightmost") == 0)
+
+  else if (cl_streq_ci(s, "rightmost"))
     return SearchRightmost;
-  else if (strcasecmp(s, "nearest") == 0)
+
+  else if (cl_streq_ci(s, "nearest"))
     return SearchNearest;
-  else if (strcasecmp(s, "farthest") == 0)
+
+  else if (cl_streq_ci(s, "farthest"))
     return SearchFarthest;
+
   else {
     cqpmessage(Warning, "Illegal search strategy specification ``%s''", s);
     return SearchNone;
@@ -57,228 +64,165 @@ SearchStrategy string_to_strategy(char *s)
 
 /* Handling of target, match, keyword. Tue Feb 28 16:02:03 1995 (oli) */
 
-/* target can be any field except NoField (-> CQP dies), 
+/* target can be any field except NoField (-> CQP dies),
    source can be NoField, which deletes the target field (unless that's match or matchend) */
+/**
+ * Sets one of the anchors (aka fields) of the NQR/subcorpus to the value of one of the others, optionally adding an token offset.
+ * It implements the CQP commands
+ *   set NQR <anchor> <anchor>[<offset>];
+ *   set NQR <anchor> NULL;
+ *
+ * There are several special cases to be considered:
+ *  - if source anchor has an undefined value (-1), the destination will either be unchanged (overwrite=0) or become undefined, too (overwrite=1)
+ *  - if source anchor's offset puts it outside the valid cpos range, it is treated as undefined (and the rule above applies)
+ *  - if destination is MatchField or MatchendField, the matching range is modified and may become invalid
+ *    (either because the destination anchor is set to undefined or because matchend ends up to the left of match);
+ *    in this case, the modification is either ignored (overwrite=0) or the match is discarded from the query (overwrite=1)
+ *
+ * @param corp          CorpusList entry for the subcorpus or NQR to modify.
+ * @param dest          Which anchor to modify (MatchField, MatchEndField, TargetField, KeywordField).
+ *                      If this is MatchField or MatchendField, the modified query will be re-sorted and some matches may be discarded.
+ *                      If this is NoField, CQP dies, since that's a logic error.
+ * @param source        Which anchor to get the data from. If NoField, the destination TargetField or KeywordField is deleted.
+ * @param source_offset Token offset to add to the source anchor(to shift anchor to the left or right).
+ * @param overwrite     Whether undefined or invalid source anchor should overwrite destination or be ignored (see above).
+ *                      In other words, overwrite=0 performs a conditional update while overwrite=1 performs a forced update.
+ *
+ * @return              True if successful, False (with a CQP error message) otherwise.
+ */
 int
-set_target(CorpusList *corp, FieldType t_id, FieldType s_id)
+set_target(CorpusList *corp, FieldType dest, FieldType source, int source_offset, int overwrite)
 {
-  int i;
+  int i, cpos, size, n_tokens;
+  cpos = -1;
 
-  if (t_id == s_id) {
-    cqpmessage(Error, "Fields are identical.");
-    return 0;
+  assert(dest != NoField);
+  if (dest == source && source_offset == 0) {
+    cqpmessage(Warning, "Anchors are identical, nothing to be done.");
+    return 1;
   }
 
-  if (corp->size == 0) {
-    cqpmessage(Error, "Corpus is empty, nothing to be done.");
-    return 0;
+  size = corp->size;
+  if (size == 0) {
+    cqpmessage(Warning, "Query result is empty, nothing to be done.");
+    return 1;
   }
   assert(corp->range);
 
+  n_tokens = corp->mother_size; /* for boundary checks if source_offset != 0 */
 
-  switch (s_id) {
-  case NoField:
-    switch (t_id) {
-    case MatchField:
-    case MatchEndField:
-      cqpmessage(Error, "Can't delete match or matchend field from %s\n", corp->name);
-      break;
-
-    case TargetField:
+  /* set NQR (keyword|target) NULL; --> delete keyword or target anchor */
+  if (source == NoField) {
+    if (dest == MatchField || dest == MatchEndField) {
+      cqpmessage(Error, "Can't delete match or matchend anchor from %s\n", corp->name);
+      return 0;
+    }
+    if (dest == TargetField)
       cl_free(corp->targets);
-      break;
-
-    case KeywordField:
+    else if (dest == KeywordField)
       cl_free(corp->keywords);
-      break;
-      
-    case NoField:
-    default:
+    else
       assert(0 && "Can't be");
-      break;
-    }
-    break;
-
-  case KeywordField:
-    if (corp->keywords == NULL)
-      cqpmessage(Error, "No keyword defined for %s\n", corp->name);
-    else {
-
-      switch (t_id) {
-      case MatchField:
-        if (corp->range == NULL) {
-          cqpmessage(Error, "Internal error: match ranges not allocated. Abort.");
-          return 0;
-        }
-
-        for (i = 0; i < corp->size; i++) {
-          if (corp->keywords[i] >= 0)
-            corp->range[i].start = corp->keywords[i];
-          if (corp->range[i].start > corp->range[i].end)
-            corp->range[i].start = corp->range[i].end;
-        }
-        break;
-
-      case MatchEndField:
-        if (corp->range == NULL) {
-          cqpmessage(Error, "Internal error: match ranges not allocated. Abort.");
-          return 0;
-        }
-
-        for (i = 0; i < corp->size; i++) {
-          if (corp->keywords[i] >= 0)
-            corp->range[i].end   = corp->keywords[i];
-          if (corp->range[i].end < corp->range[i].start)
-            corp->range[i].end = corp->range[i].start;
-        }
-        break;
-
-      case TargetField:
-        if (corp->targets == NULL)
-          corp->targets = (int *)cl_malloc(corp->size * sizeof(int));
-        /* bcopy(corp->keywords, corp->targets, corp->size * sizeof(int)); */
-        memcpy(corp->targets, corp->keywords, corp->size * sizeof(int));
-        break;
-
-      case NoField:
-      default:
-        assert(0 && "Can't be");
-        break;
-      }
-
-    }
-    break;
-
-  case TargetField:
-    if (corp->targets == NULL)
-      cqpmessage(Error, "No collocates / targets defined for %s\n", corp->name);
-    else {
-
-      switch (t_id) {
-      case MatchField:
-        if (corp->range == NULL) {
-          cqpmessage(Error, "Internal error: match ranges not allocated. Abort.");
-          return 0;
-        }
-
-        for (i = 0; i < corp->size; i++) {
-          if (corp->targets[i] >= 0)
-            corp->range[i].start = corp->targets[i];
-          if (corp->range[i].start > corp->range[i].end)
-            corp->range[i].start = corp->range[i].end;
-        }
-        break;
-
-      case MatchEndField:
-        if (corp->range == NULL) {
-          cqpmessage(Error, "Internal error: match ranges not allocated. Abort.");
-          return 0;
-        }
-
-        for (i = 0; i < corp->size; i++) {
-          if (corp->targets[i] >= 0)
-            corp->range[i].end   = corp->targets[i];
-          if (corp->range[i].end < corp->range[i].start)
-            corp->range[i].end = corp->range[i].start;
-        }
-        break;
-
-      case KeywordField:
-        if (corp->keywords == NULL)
-          corp->keywords = (int *)cl_malloc(corp->size * sizeof(int));
-        /* bcopy(corp->targets, corp->keywords, corp->size * sizeof(int)); */
-        memcpy(corp->keywords, corp->targets, corp->size * sizeof(int));
-        break;
-
-      case NoField:
-      default:
-        assert(0 && "Can't be");
-        break;
-      }
-
-    }
-    break;
-
-  case MatchField:
-
-    switch (t_id) {
-    case MatchEndField:
-      for (i = 0; i < corp->size; i++)
-        corp->range[i].end = corp->range[i].start;
-      break;
-
-    case KeywordField:
-      if (corp->keywords == NULL)
-        corp->keywords = (int *)cl_malloc(corp->size * sizeof(int));
-      for (i = 0; i < corp->size; i++)
-        corp->keywords[i] = corp->range[i].start;
-      break;
-
-    case TargetField:
-      if (corp->targets == NULL)
-        corp->targets = (int *)cl_malloc(corp->size * sizeof(int));
-      for (i = 0; i < corp->size; i++)
-        corp->targets[i] = corp->range[i].start;
-      break;
-
-    case NoField:
-    default:
-      assert(0 && "Can't be");
-      break;
-    }
-
-    break;
-
-  case MatchEndField:
-
-    switch (t_id) {
-    case MatchField:
-      for (i = 0; i < corp->size; i++)
-        corp->range[i].start = corp->range[i].end;
-      break;
-      
-    case KeywordField:
-      if (corp->keywords == NULL)
-        corp->keywords = (int *)cl_malloc(corp->size * sizeof(int));
-      for (i = 0; i < corp->size; i++)
-        corp->keywords[i] = corp->range[i].end;
-      break;
-
-    case TargetField:
-      if (corp->targets == NULL)
-        corp->targets = (int *)cl_malloc(corp->size * sizeof(int));
-      for (i = 0; i < corp->size; i++)
-        corp->targets[i] = corp->range[i].end;
-      break;
-
-    case NoField:
-    default:
-      assert(0 && "Can't be");
-      break;
-    }
-    break;
-
-  default:
-    assert("Can't be" && 0);
-    break;
   }
-  
-  if ((t_id == MatchField) || (t_id == MatchEndField))
-    RangeSort(corp, 0);                /* re-sort corpus if match regions were modified */
-  touch_corpus(corp);
+  /* set NQR <anchor> <anchor>[<offset>]; --> update destination anchor from source */
+  else {
+    if (source == TargetField && !corp->targets) {
+      cqpmessage(Error, "No target anchors defined for %s\n", corp->name);
+      return 0;
+    }
+    if (source == KeywordField && !corp->keywords) {
+      cqpmessage(Error, "No keyword anchors defined for %s\n", corp->name);
+      return 0;
+    }
+    /* having handled all possible error conditions, we can now safely allocate anchor vectors if needed */
+    if (dest == TargetField && !corp->targets) {
+      corp->targets = (int *)cl_malloc(size * sizeof(int));
+      for (i = 0; i < size; i++)
+        corp->targets[i] = -1; /* initialise to undefined values */
+    }
+    if (dest == KeywordField && !corp->keywords) {
+      corp->keywords = (int *)cl_malloc(size * sizeof(int));
+      for (i = 0; i < size; i++)
+        corp->keywords[i] = -1;
+    }
 
+    /* In a textbook case of premature optimisation, the previous implementation had 16 different versions of the loop below,
+     * for each possible combination of source and destination anchor. This was done to avoid case distinctions in the inner
+     * loop and even replace it with a memcopy() in some cases. The optimisation was a completely unnecessary complication of
+     * the source code because
+     * 1) the set target command is invoked only occasionally, and even with a suboptimal implementation it will be much faster
+     *    than the original query, requiring only a single linear pass through the result set;
+     * 2) more importantly, if-clauses within the inner loop always take the same branch (based on source and dest), which the
+     *    CPU's branch prediction will learn quickly, reducing the overhead from the case distinctions to be quite irrelevant.
+     * Hence, the new implementation uses a single outer loop over the result set and handles the different source and destination
+     * anchors within the loop, thus avoiding any code duplication and making the algorithm much more transparent.
+     */
+    for (i = 0; i < size; i++) {
+      /* get the source anchor for this match */
+      if (source == MatchField)
+        cpos = corp->range[i].start;
+      else if (source == MatchEndField)
+        cpos = corp->range[i].end;
+      else if (source == TargetField)
+        cpos = corp->targets[i];
+      else if (source == KeywordField)
+        cpos = corp->keywords[i];
+      else
+        assert(0 && "Can't be");
+
+      /* add the offset and check for range overflow */
+      if (source_offset != 0 && cpos >= 0) {
+        cpos += source_offset;
+        if (cpos < 0 || cpos >= n_tokens)
+          cpos = -1; /* if the anchor is shifted outside the corpus, it becomes undefined */
+      }
+
+      /* assign to target or keyword */
+      if (dest == TargetField) {
+        if (overwrite || cpos >= 0)
+          corp->targets[i] = cpos;
+      }
+      else if (dest == KeywordField) {
+        if (overwrite || cpos >= 0)
+          corp->keywords[i] = cpos;
+      }
+      else if (dest == MatchField) {
+        if (cpos >= 0 && cpos <= corp->range[i].end)
+          corp->range[i].start = cpos;
+        else if (overwrite)
+          corp->range[i].start = -1; /* mark for deletion if overwriting makes match invalid */
+      }
+      else if (dest == MatchEndField) {
+        if (cpos >= 0 && cpos >= corp->range[i].start)
+          corp->range[i].end = cpos;
+        else if (overwrite)
+          corp->range[i].start = -1; /* note that we have to set the match anchor to -1 to mark for deletion */
+      }
+      else
+        assert(0 && "Can't be");
+    }
+
+    /* re-sort corpus and discard invalid matching spans if match or matchend anchor was modified */
+    if (dest == MatchField || dest == MatchEndField) {
+      apply_range_set_operation(corp, RReduce, NULL, NULL);
+      RangeSort(corp, 0);
+    }
+  }
+
+  touch_corpus(corp);
   return 1;
 }
 
-int evaluate_target(CorpusList *corp,          /* the corpus */
-                    FieldType t_id,            /* the field to set */
-                    FieldType base,            /* where to start the search */
-                    int inclusive,             /* including or excluding the base */
-                    SearchStrategy strategy,   /* disambiguation rule: which item */
-                    Constrainttree constr,     /* the constraint */
-                    enum ctxtdir direction,    /* context direction */
-                    int units,                       /* number of units */
-                    char *attr_name)           /* name of unit */
+int
+evaluate_target(CorpusList *corp,          /* the corpus */
+                FieldType t_id,            /* the field to set */
+                FieldType base,            /* where to start the search */
+                int inclusive,             /* including or excluding the base */
+                SearchStrategy strategy,   /* disambiguation rule: which item */
+                Constrainttree constr,     /* the constraint */
+                enum ctxtdir direction,    /* context direction */
+                int units,                 /* number of units */
+                char *attr_name)           /* name of unit */
 {
   Attribute *attr;
   int *table;
@@ -296,7 +240,7 @@ int evaluate_target(CorpusList *corp,          /* the corpus */
   assert(t_id == TargetField || t_id == KeywordField || t_id == MatchField || t_id == MatchEndField);
 
   if (!constr) {
-    cqpmessage(Error, "Constraing pattern missing in 'set target' command.");
+    cqpmessage(Error, "Constraint pattern missing in 'set target' command.");
     return 0;
   }
 
@@ -305,10 +249,9 @@ int evaluate_target(CorpusList *corp,          /* the corpus */
     return 0;
   }
 
-  /*
-   * check whether the base field specification is ok
-   */
+  /* check whether the base field specification is ok */
   switch(base) {
+
   case MatchField:
   case MatchEndField:
     if (corp->range == NULL) {
@@ -316,27 +259,28 @@ int evaluate_target(CorpusList *corp,          /* the corpus */
       return 0;
     }
     break;
+
   case TargetField:
     if (corp->targets == NULL) {
       cqpmessage(Error, "Can't start from base TARGET, none defined");
       return 0;
     }
     break;
+
   case KeywordField:
     if (corp->keywords == NULL) {
       cqpmessage(Error, "Can't start from base KEYWORD, none defined");
       return 0;
     }
     break;
+
   default:
-    cqpmessage(Error, "Illegal base field (#%d) in 'set target' command.",
-               base);
+    cqpmessage(Error, "Illegal base field (#%d) in 'set target' command.", base);
     return 0;
   }
 
   if (units <= 0) {
-    cqpmessage(Error, "Invalid search space (%d units) in 'set target' command.", 
-               units);
+    cqpmessage(Error, "Invalid search space (%d units) in 'set target' command.", units);
     return 0;
   }
 
@@ -349,21 +293,19 @@ int evaluate_target(CorpusList *corp,          /* the corpus */
   context.size = units;
   context.direction = direction;
 
-  if ((strcasecmp(attr_name, "word") == 0) ||
-      (strcasecmp(attr_name, "words") == 0)) {
-    attr = find_attribute(corp->corpus, DEFAULT_ATT_NAME, ATT_POS, NULL);
-    context.type = word;
+  if (cl_streq_ci(attr_name, "word") || cl_streq_ci(attr_name, "words")) {
+    attr = cl_new_attribute(corp->corpus, CWB_DEFAULT_ATT_NAME, ATT_POS);
+    context.space_type = word;
     context.attrib = NULL;
   }
   else {
-    attr = find_attribute(corp->corpus, attr_name, ATT_STRUC, NULL);
-    context.type = structure;
+    attr = cl_new_attribute(corp->corpus, attr_name, ATT_STRUC);
+    context.space_type = structure;
     context.attrib = attr;
   }
 
   if (attr == NULL) {
-    cqpmessage(Error, "Can't find attribute %s.%s",
-               corp->mother_name, attr_name);
+    cqpmessage(Error, "Can't find attribute %s.%s", corp->mother_name, attr_name);
     return 0;
   }
 
@@ -372,7 +314,6 @@ int evaluate_target(CorpusList *corp,          /* the corpus */
     progress_bar_message(1, 1, "    preparing");
   }
 
-
   table = (int *)cl_calloc(corp->size, sizeof(int));
 
   EvaluationIsRunning = 1;
@@ -392,35 +333,34 @@ int evaluate_target(CorpusList *corp,          /* the corpus */
     table[line] = -1;
 
     switch(base) {
-    case MatchField:
 
+    case MatchField:
       excl_start = corp->range[line].start;
       excl_end   = corp->range[line].end;
 
       if ((corp->range[line].start == corp->range[line].end) || inclusive) {
 
-        if (calculate_ranges(corp,
-                             corp->range[line].start, context,
-                             &lbound, &rbound) == False) {
+        if (!calculate_ranges(corp,
+                              corp->range[line].start, context,
+                              &lbound, &rbound)) {
 
           Rprintf("Can't compute boundaries for range #%d", line);
           lbound = rbound = -1;
         }
       }
       else {
-
         int dummy;
 
-        if (calculate_ranges(corp,
+        if (!calculate_ranges(corp,
                              corp->range[line].start, context,
-                             &lbound, &dummy) == False) {
+                             &lbound, &dummy)) {
 
           Rprintf("Can't compute left search space boundary match #%d", line);
           lbound = rbound = -1;
         }
-        else if (calculate_ranges(corp,
+        else if (!calculate_ranges(corp,
                                   corp->range[line].end, context,
-                                  &dummy, &rbound) == False) {
+                                  &dummy, &rbound)) {
 
           Rprintf("Can't compute right search space boundary match #%d", line);
           lbound = rbound = -1;
@@ -432,69 +372,66 @@ int evaluate_target(CorpusList *corp,          /* the corpus */
       excl_start = excl_end = corp->range[line].end;
 
       if (excl_start >= 0) {
-        if (calculate_ranges(corp,
-                             corp->range[line].end, context,
-                             &lbound, &rbound) == False) {
+        if (!calculate_ranges(corp,
+                              corp->range[line].end, context,
+                              &lbound, &rbound)) {
 
           Rprintf("Can't compute search space boundaries for match #%d", line);
           lbound = rbound = -1;
         }
       }
-      else 
+      else
         lbound = rbound = -1;
-
       break;
 
     case TargetField:
       excl_start = excl_end = corp->targets[line];
 
       if (excl_start >= 0) {
-        if (calculate_ranges(corp,
-                             corp->targets[line], context,
-                                  &lbound, &rbound) == False) {
+        if (!calculate_ranges(corp,
+                              corp->targets[line], context,
+                              &lbound, &rbound)) {
 
           Rprintf("Can't compute search space boundaries for match #%d", line);
           lbound = rbound = -1;
         }
       }
-      else 
+      else
         lbound = rbound = -1;
-
       break;
 
     case KeywordField:
       excl_start = excl_end = corp->keywords[line];
 
       if (excl_start >= 0) {
-        if (calculate_ranges(corp,
-                             corp->keywords[line], context,
-                             &lbound, &rbound) == False) {
+        if (!calculate_ranges(corp,
+                              corp->keywords[line], context,
+                              &lbound, &rbound)) {
 
           Rprintf("Can't compute search space boundaries for match #%d", line);
           lbound = rbound = -1;
         }
       }
-      else 
+      else
         lbound = rbound = -1;
-
       break;
+
     default:
       assert(0 && "Can't be");
       return 0;
     }
 
-    if ((lbound >= 0) && (rbound >= 0)) {
-      
+    if (lbound >= 0 && rbound >= 0) {
       int dist, maxdist;
 
-      if (direction == left) {
+      if (direction == ctxtdir_left) {
         rbound = excl_start;
         if (strategy == SearchNearest)
           strategy = SearchRightmost;
         else if (strategy == SearchFarthest)
           strategy = SearchLeftmost;
       }
-      else if (direction == right) {
+      else if (direction == ctxtdir_right) {
         lbound = excl_start;
         if (strategy == SearchNearest)
           strategy = SearchLeftmost;
@@ -503,43 +440,35 @@ int evaluate_target(CorpusList *corp,          /* the corpus */
       }
 
       switch (strategy) {
+
       case SearchFarthest:
-
         maxdist = MAX(excl_start - lbound, rbound - excl_start);
-
         assert(maxdist >= 0);
 
         for (dist = maxdist; dist >= 0; dist--) {
 
           i = excl_start - dist;
-
-          if (i >= lbound &&
-              (inclusive || (i < excl_start)))
+          if (i >= lbound && (inclusive || i < excl_start) )
             if (eval_bool(constr, NULL, i)) {
               table[line] = i;
               break;
             }
 
           i = excl_start + dist;
-
-          if (i <= rbound &&
-              (inclusive || (i > excl_end)))
+          if (i <= rbound && (inclusive || i > excl_end))
             if (eval_bool(constr, NULL, i)) {
               table[line] = i;
               break;
             }
 
-          nr_evals++;
-          if (nr_evals == 1000) {
+          if (++nr_evals == 1000) {
             CheckForInterrupts();
             nr_evals = 0;
           }
-
         }
         break;
 
       case SearchNearest:
-
         maxdist = MAX(excl_start - lbound, rbound - excl_start);
         assert(maxdist >= 0);
 
@@ -547,8 +476,7 @@ int evaluate_target(CorpusList *corp,          /* the corpus */
 
           i = excl_start - dist;
 
-          if (i >= lbound &&
-              (inclusive || (i < excl_start)))
+          if (i >= lbound && (inclusive || (i < excl_start)))
             if (eval_bool(constr, NULL, i)) {
               table[line] = i;
               break;
@@ -556,19 +484,16 @@ int evaluate_target(CorpusList *corp,          /* the corpus */
 
           i = excl_start + dist;
 
-          if (i <= rbound &&
-              (inclusive || (i > excl_end)))
+          if (i <= rbound && (inclusive || (i > excl_end)))
             if (eval_bool(constr, NULL, i)) {
               table[line] = i;
               break;
             }
 
-          nr_evals++;
-          if (nr_evals == 1000) {
+          if (++nr_evals == 1000) {
             CheckForInterrupts();
             nr_evals = 0;
           }
-
         }
         break;
 
@@ -580,8 +505,7 @@ int evaluate_target(CorpusList *corp,          /* the corpus */
               break;
             }
 
-            nr_evals++;
-            if (nr_evals == 1000) {
+            if (++nr_evals == 1000) {
               CheckForInterrupts();
               nr_evals = 0;
             }
@@ -596,26 +520,26 @@ int evaluate_target(CorpusList *corp,          /* the corpus */
               break;
             }
 
-            nr_evals++;
-            if (nr_evals == 1000) {
+            if (++nr_evals == 1000) {
               CheckForInterrupts();
               nr_evals = 0;
             }
           }
         break;
+
       default:
         break;
       }
     }
   }
 
-  if (progress_bar) 
+  if (progress_bar)
     progress_bar_message(1, 1, "  cleaning up");
 
   switch (t_id) {
   case MatchField:
     for (i = 0; i < corp->size; i++) {
-      if (table[i] >= 0) 
+      if (table[i] >= 0)
         corp->range[i].start = table[i];
       if (corp->range[i].start > corp->range[i].end)
         corp->range[i].start = corp->range[i].end;
@@ -625,7 +549,7 @@ int evaluate_target(CorpusList *corp,          /* the corpus */
 
   case MatchEndField:
     for (i = 0; i < corp->size; i++) {
-      if (table[i] >= 0) 
+      if (table[i] >= 0)
         corp->range[i].end = table[i];
       if (corp->range[i].end < corp->range[i].start)
         corp->range[i].end = corp->range[i].start;
@@ -651,22 +575,30 @@ int evaluate_target(CorpusList *corp,          /* the corpus */
   if (progress_bar)
     progress_bar_clear_line();
 
-  if ((t_id == MatchField) || (t_id == MatchEndField))
-    RangeSort(corp, 0);                /* re-sort corpus if match regions were modified */
+  /* re-sort corpus if match regions were modified */
+  if (t_id == MatchField || t_id == MatchEndField)
+    RangeSort(corp, 0);
+
+  touch_corpus(corp);
 
-  touch_corpus(corp);  
   if (!EvaluationIsRunning) {
     cqpmessage(Warning, "Evaluation interruted: results may be incomplete.");
-    if (which_app == cqp) install_signal_handler();
+    if (which_app == cqp)
+      install_signal_handler();
   }
   EvaluationIsRunning = 0;
 
   return 1;
 }
 
-int evaluate_subset(CorpusList *cl, /* the corpus */
-                    FieldType the_field,       /* the field to scan */
-                    Constrainttree constr)
+/**
+ * @param cl            the corpus
+ * @param the_field     the field to scan
+ * @param constr
+ * @return
+ */
+int
+evaluate_subset(CorpusList *cl, FieldType the_field, Constrainttree constr)
 {
   int line, position;
   int percentage, new_percentage; /* for ProgressBar */
@@ -676,8 +608,7 @@ int evaluate_subset(CorpusList *cl, /* the corpus */
 
   percentage = -1;
 
-  EvaluationIsRunning = 1;
-  for (line = 0; (line < cl->size) && EvaluationIsRunning; line++) {
+  for (EvaluationIsRunning = 1, line = 0; line < cl->size && EvaluationIsRunning; line++) {
 
     if (progress_bar) {
       new_percentage = floor(0.5 + (100.0 * line) / cl->size);
@@ -688,25 +619,25 @@ int evaluate_subset(CorpusList *cl, /* the corpus */
     }
 
     switch (the_field) {
-    
+
     case MatchField:
       position = cl->range[line].start;
       break;
-      
+
     case MatchEndField:
       position = cl->range[line].end;
       break;
-      
+
     case KeywordField:
       assert(cl->keywords);
       position = cl->keywords[line];
       break;
-      
+
     case TargetField:
       assert(cl->targets);
       position = cl->targets[line];
       break;
-      
+
     case NoField:
     default:
       position = -1;
@@ -718,7 +649,7 @@ int evaluate_subset(CorpusList *cl, /* the corpus */
       cl->range[line].end   = -1;
     }
   }
-  
+
   /* if interrupted, delete part of temporary query result which hasn't been filtered;
      so that the result is incomplete but at least contains only correct matches */
   while (line < cl->size) {
@@ -729,14 +660,15 @@ int evaluate_subset(CorpusList *cl, /* the corpus */
 
   if (!EvaluationIsRunning) {
     cqpmessage(Warning, "Evaluation interruted: results may be incomplete.");
-    if (which_app == cqp) install_signal_handler();
+    if (which_app == cqp)
+      install_signal_handler();
   }
   EvaluationIsRunning = 0;
 
-  if (progress_bar) 
+  if (progress_bar)
     progress_bar_message(0, 0, "  cleaning up");
 
-  (void) RangeSetop(cl, RReduce, NULL, NULL);
+  apply_range_set_operation(cl, RReduce, NULL, NULL);
 
   return 1;
 }
