@@ -1,13 +1,13 @@
-/* 
+/*
  *  IMS Open Corpus Workbench (CWB)
  *  Copyright (C) 1993-2006 by IMS, University of Stuttgart
  *  Copyright (C) 2007-     by the respective contributers (see file AUTHORS)
- * 
+ *
  *  This program is free software; you can redistribute it and/or modify it
  *  under the terms of the GNU General Public License as published by the
  *  Free Software Foundation; either version 2, or (at your option) any later
  *  version.
- * 
+ *
  *  This program is distributed in the hope that it will be useful, but
  *  WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
@@ -17,16 +17,18 @@
 
 #include <stdio.h>
 #include <stdlib.h>
+
 #include <sys/types.h>
 
+#include "../cl/cl.h"
+#include "../cl/cwb-globals.h"
 /* byte order handling taken from Corpus Library */
 #include "../cl/endian.h"
-#include "../cl/cl.h"
 
 /* LONG and SHORT modes removed. Mon Mar 23 19:25:35 MET 1998 (evert) */
 
 
-int little_endian = 0;  /* CWB default format is 4-byte big-endian = network */
+int little_endian = 0;  /* boolean; CWB default format is 4-byte big-endian = network */
 
 int buf[CL_MAX_LINE_LENGTH];
 
@@ -34,30 +36,28 @@ int buf[CL_MAX_LINE_LENGTH];
  * Reads one integer at a time from a stream and prints a decimal representation
  * of it on STDOUT; strings representing ints are split by newlines.
  *
- * @param fd  The file handle.
+ * @param stream  The file handle.
  */
 void
-process_fd(FILE *fd)
+itoa_process_stream(FILE *stream)
 {
   int N, k, i;
 
   do {
     /* currently only works on systems with 32bit ints.
        should really be fixed some time */
-    N = fread(&buf[0], sizeof(int), CL_MAX_LINE_LENGTH, fd);
+    N = fread(&buf[0], sizeof(int), CL_MAX_LINE_LENGTH, stream);
 
-    for ( k = 0; k < N; k++) {
+    for (k = 0; k < N; k++) {
       i = ntohl(buf[k]);        /* convert from CWB to internal format */
-      if (little_endian) 
+      if (little_endian)
         i = cl_bswap32(i);      /* explicit conversion */
       fprintf(stdout, "%d\n", i);
     }
   } while (N == CL_MAX_LINE_LENGTH);
 }
 
-/* *************** *\
- *      MAIN()     *
-\* *************** */
+
 
 /**
  * Main function for cwb-itoa.
@@ -68,11 +68,13 @@ process_fd(FILE *fd)
 int
 main(int argc, char **argv)
 {
-  FILE *fd;
+  FILE *src;
   int i;
   char *progname = argv[0];
 
-  fd = stdin;  /* initialisation removed from declaration for Gnuwin32 compatibility */
+  cl_startup();
+
+  src = stdin;  /* initialisation removed from declaration for Gnuwin32 compatibility */
 
   for (i = 1; i < argc; i++) {
     if (argv[i][0] == '-') {
@@ -93,17 +95,17 @@ main(int argc, char **argv)
         fprintf(stderr, "Options:\n");
         fprintf(stderr, "  -n  read integers in network format [default]\n");
         fprintf(stderr, "  -l  read integers in little endian format\n");
-        fprintf(stderr, "Part of the IMS Open Corpus Workbench v" VERSION "\n\n");
+        fprintf(stderr, "Part of the IMS Open Corpus Workbench v" CWB_VERSION "\n\n");
         exit(1);
       }
     }
-    else if ((fd = fopen(argv[i], "rb")) == NULL) {
+    else if ((src = fopen(argv[i], "rb")) == NULL) {
       fprintf(stderr, "%s: Couldn't open %s\n", progname, argv[i]);
       exit(1);
     }
   }
 
   /* now process either input file or stdin */
-  process_fd(fd);
+  itoa_process_stream(src);
   return 0;
 }
